/*
 Copyright (c) 2009 [Joerg Ruedenauer]
 
 This file is part of RPGSoundController.

 RPGSoundController is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 RPGSoundController is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with RPGSoundController; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package rpgsoundcontroller.control;

import java.util.prefs.Preferences;

import javax.swing.SwingUtilities;

import rpgsoundcontroller.gui.MainFrame;
import rpgsoundcontroller.gui.SubFrame;
import rpgsoundcontroller.gui.lf.LookAndFeels;

/**
 * 
 */
public class Main {
  
  private Main() {}

  private static java.util.List<String> startFiles = null;
  
  public static void main(String[] args) {
    SubFrame.setSaveLocations(false);
    if (args.length > 0) {
      startFiles = java.util.Arrays.asList(args);
    }
    else {
      String lastFile = Preferences.userNodeForPackage(MainFrame.class).get("LastConfiguration", ""); //$NON-NLS-1$ //$NON-NLS-2$
      if (lastFile != null && lastFile.length() != 0) {
        startFiles = new java.util.ArrayList<String>();
        startFiles.add(lastFile);
      }
    }

    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        createAndShowGUI();
      }
    });
  }
  
  public static void exit(int retCode)
  {
    System.exit(retCode);
  }

  private static void createAndShowGUI() {
    try {
      LookAndFeels.setLastLookAndFeel();
      MainFrame mainFrame = new MainFrame();
      mainFrame.setVisible(true);
      if (startFiles != null) {
        mainFrame.openFile(startFiles.get(0));
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }
}
