/*
 Copyright (c) 2009 [Joerg Ruedenauer]
 
 This file is part of RPGSoundController.

 RPGSoundController is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 RPGSoundController is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with RPGSoundController; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package rpgsoundcontroller.data;

import java.util.ArrayList;
import java.util.List;

import javax.swing.KeyStroke;

import rpgsoundcontroller.util.Localization;

public final class Mode extends Command {
  
  private List<Command> commands;

  public Mode(String title, KeyStroke keyStroke) {
    super(title, keyStroke, true, -1, false);
    commands = new ArrayList<Command>();
  }
  
  public void addCommand(Command command) {
    if (containsKeyStroke(command.getKeyStroke())) throw new IllegalArgumentException(Localization.getString("Mode.DuplicateKey")); //$NON-NLS-1$
    commands.add(command);
  }
  
  public List<Command> getCommands() {
    return java.util.Collections.unmodifiableList(commands);
  }
  
  public boolean containsKeyStroke(KeyStroke keyStroke) {
    for(Command command : commands) {
      if (command.getKeyStroke().equals(keyStroke)) return true;
    }
    return false;
  }
  
}
