/*
 Copyright (c) 2009 [Joerg Ruedenauer]
 
 This file is part of RPGSoundController.

 RPGSoundController is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 RPGSoundController is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with RPGSoundController; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package rpgsoundcontroller.gui;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JToggleButton;

import rpgsoundcontroller.control.Control;
import rpgsoundcontroller.data.Command;

abstract class CommandsFrame extends FrameController {
  
  public CommandsFrame(String title, List<Command> commands) {
    super(title);
    this.commands = commands;
    buttonGroups = new HashMap<Integer, List<AbstractButton>>();
    Control.getInstance().addAlwaysAvailableKeys(getRootPane());
  }
  
  protected void initialize() {
    this.setContentPane(getJContentPane());
    this.setDefaultCloseOperation(javax.swing.JFrame.DISPOSE_ON_CLOSE);                 
    pack();
  }
  
  private static class KeyCommandAction extends AbstractAction {
    
    private AbstractButton button;
    
    public KeyCommandAction(AbstractButton button) {
      this.button = button;
    }

    public void actionPerformed(ActionEvent arg0) {
      button.doClick();
    }
    
  }
  
  /**
   * This method initializes jContentPane 
   *  
   * @return javax.swing.JPanel 
   */
  private JPanel getJContentPane() {
    if (jContentPane == null) {
      jContentPane = new JPanel();
      jContentPane.setLayout(new BorderLayout());
      JLabel label1 = new JLabel();
      label1.setPreferredSize(new java.awt.Dimension(5, 5));
      jContentPane.add(label1, BorderLayout.NORTH);
      JLabel label2 = new JLabel();
      label2.setPreferredSize(new java.awt.Dimension(5, 5));
      jContentPane.add(label2, BorderLayout.EAST);
      JLabel label3 = new JLabel();
      label3.setPreferredSize(new java.awt.Dimension(5, 5));
      jContentPane.add(label3, BorderLayout.SOUTH);
      JLabel label4 = new JLabel();
      label4.setPreferredSize(new java.awt.Dimension(5, 5));
      jContentPane.add(label4, BorderLayout.WEST);
      JPanel buttonPane = new JPanel();
      buttonPane.setLayout(new GridLayout(Math.round(commands.size() / 2.0f), 2, 10, 10));
      for (Command command : commands) {
        AbstractButton button = command.isContinuous() ? new JToggleButton() : new JButton();
        AbstractAction action = createAction(command, button); 
        button.setAction(action);
        button.setText(command.getTitle());
        if (command.isContinuous()) {
          if (!buttonGroups.containsKey(command.getGroup())) {
            buttonGroups.put(command.getGroup(), new ArrayList<AbstractButton>());
          }
          buttonGroups.get(command.getGroup()).add(button);
          button.addActionListener(new GroupUpdater(command.getGroup()));
        }
        buttonPane.add(button);
        getRootPane().getActionMap().put(command.getTitle(), new KeyCommandAction(button));
        getRootPane().getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(command.getKeyStroke(), command.getTitle());
      }
      jContentPane.add(buttonPane, BorderLayout.CENTER);
    }
    return jContentPane;
  }
  
  private class GroupUpdater implements ActionListener {
    public GroupUpdater(int group) {
      this.group = group;
    }
    private int group;
    public void actionPerformed(ActionEvent e) {
      if (!((AbstractButton)e.getSource()).isSelected()) return;
      for (AbstractButton button : buttonGroups.get(group)) {
        if (button != e.getSource()) {
          deselectButton(button);
        }
      }
    }
  }

  protected abstract AbstractAction createAction(Command command, AbstractButton button);
  
  protected abstract void deselectButton(AbstractButton button);
  
  private JPanel jContentPane;

  private List<Command> commands;
  
  private Map<Integer, List<AbstractButton>> buttonGroups;
}
