/*
 Copyright (c) 2009 [Joerg Ruedenauer]
 
 This file is part of RPGSoundController.

 RPGSoundController is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 RPGSoundController is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with RPGSoundController; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package rpgsoundcontroller.gui;

import java.awt.Dimension;
import javax.swing.JPanel;
import javax.swing.BorderFactory;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.KeyEvent;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.KeyStroke;

import rpgsoundcontroller.control.Control;
import rpgsoundcontroller.control.KeyAction;
import rpgsoundcontroller.control.StopAllAction;
import rpgsoundcontroller.gui.util.RepeatingButtonModel;
import rpgsoundcontroller.util.Localization;


class MainControlsFrame extends SubFrame {

  private JPanel jContentPane = null;
  private JPanel jPanel = null;
  private JButton nextTrackButton = null;
  private JButton previousButton = null;
  private JButton stopAllButton = null;
  private JPanel jPanel1 = null;
  private JLabel jLabel = null;
  private JLabel jLabel1 = null;
  private JLabel jLabel2 = null;
  private JButton volDownButton = null;
  private JButton volUpButton = null;
  private JButton soundsDownButton = null;
  private JButton soundsUpButton = null;
  private JButton musicDownButton = null;
  private JButton musicUpButton = null;

  /**
   * This method initializes 
   * 
   */
  public MainControlsFrame() {
  	super(Localization.getString("MainControlsFrame.MainControls")); //$NON-NLS-1$
  	initialize();
  	Control.getInstance().addAlwaysAvailableKeys(getRootPane());
  }
  
  private static KeyStroke getKeyStroke(int keyCode) {
    return KeyStroke.getKeyStroke(keyCode, 0);
  }
  
  /**
   * This method initializes this
   * 
   */
  private void initialize() {
        this.setContentPane(getJContentPane());
        this.setTitle(Localization.getString("MainControlsFrame.MainControls")); //$NON-NLS-1$
        pack();
  }

  /**
   * This method initializes jContentPane	
   * 	
   * @return javax.swing.JPanel	
   */
  private JPanel getJContentPane() {
    if (jContentPane == null) {
      jContentPane = new JPanel();
      jContentPane.setLayout(new BorderLayout());
      JLabel upper = new JLabel();
      upper.setPreferredSize(new Dimension(10, 10));
      jContentPane.add(upper, BorderLayout.NORTH);
      JLabel lower = new JLabel();
      lower.setPreferredSize(new Dimension(3, 3));
      jContentPane.add(lower, BorderLayout.SOUTH);
      JLabel right = new JLabel();
      right.setPreferredSize(new Dimension(3, 3));
      jContentPane.add(right, BorderLayout.EAST);
      JLabel left = new JLabel();
      left.setPreferredSize(new Dimension(3, 3));
      jContentPane.add(left, BorderLayout.WEST);
      JPanel center = new JPanel();
      center.setLayout(new BoxLayout(center, BoxLayout.LINE_AXIS));
      center.add(getJPanel());
      center.add(Box.createRigidArea(new Dimension(3, 0)));
      center.add(getJPanel1());
      jContentPane.add(center, BorderLayout.CENTER);
    }
    return jContentPane;
  }

  /**
   * This method initializes jPanel	
   * 	
   * @return javax.swing.JPanel	
   */
  private JPanel getJPanel() {
    if (jPanel == null) {
      jPanel = new JPanel();
      jPanel.setLayout(new BorderLayout());
      JLabel lower = new JLabel();
      lower.setPreferredSize(new Dimension(3, 3));
      jPanel.add(lower, BorderLayout.SOUTH);
      JLabel right = new JLabel();
      right.setPreferredSize(new Dimension(3, 3));
      jPanel.add(right, BorderLayout.EAST);
      JLabel left = new JLabel();
      left.setPreferredSize(new Dimension(3, 3));
      jPanel.add(left, BorderLayout.WEST);
      JPanel center = new JPanel();
      center.setLayout(new GridLayout(3, 1, 5, 5));
      jPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), Localization.getString("MainControlsFrame.Common"), TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, new Font("Dialog", Font.BOLD, 12), new Color(51, 51, 51))); //$NON-NLS-1$ //$NON-NLS-2$
      center.add(getNextTrackButton(), null);
      center.add(getPreviousButton(), null);
      center.add(getStopAllButton(), null);
      jPanel.add(center, BorderLayout.CENTER);
    }
    return jPanel;
  }

  /**
   * This method initializes nextTrackButton	
   * 	
   * @return javax.swing.JButton	
   */
  private JButton getNextTrackButton() {
    if (nextTrackButton == null) {
      nextTrackButton = new JButton();
      nextTrackButton.setAction(new KeyAction(getKeyStroke(KeyEvent.VK_RIGHT)));
      nextTrackButton.setText(Localization.getString("MainControlsFrame.NextTrack")); //$NON-NLS-1$
    }
    return nextTrackButton;
  }

  /**
   * This method initializes previousButton	
   * 	
   * @return javax.swing.JButton	
   */
  private JButton getPreviousButton() {
    if (previousButton == null) {
      previousButton = new JButton();
      previousButton.setAction(new KeyAction(getKeyStroke(KeyEvent.VK_LEFT)));
      previousButton.setText(Localization.getString("MainControlsFrame.PreviousTrack")); //$NON-NLS-1$
    }
    return previousButton;
  }

  /**
   * This method initializes stopAllButton	
   * 	
   * @return javax.swing.JButton	
   */
  private JButton getStopAllButton() {
    if (stopAllButton == null) {
      stopAllButton = new JButton();
      stopAllButton.setAction(new StopAllAction(getKeyStroke(KeyEvent.VK_ESCAPE)));
      stopAllButton.setText(Localization.getString("MainControlsFrame.StopAll")); //$NON-NLS-1$
    }
    return stopAllButton;
  }

  /**
   * This method initializes jPanel1	
   * 	
   * @return javax.swing.JPanel	
   */
  private JPanel getJPanel1() {
    if (jPanel1 == null) {
      jLabel2 = new JLabel();
      jLabel2.setText(Localization.getString("MainControlsFrame.Music")); //$NON-NLS-1$
      jLabel1 = new JLabel();
      jLabel1.setText(Localization.getString("MainControlsFrame.Sounds")); //$NON-NLS-1$
      jLabel = new JLabel();
      jLabel.setText(Localization.getString("MainControlsFrame.Overall")); //$NON-NLS-1$
      jPanel1 = new JPanel();
      jPanel1.setLayout(new BorderLayout());
      JLabel lower = new JLabel();
      lower.setPreferredSize(new Dimension(3, 3));
      jPanel1.add(lower, BorderLayout.SOUTH);
      JLabel right = new JLabel();
      right.setPreferredSize(new Dimension(3, 3));
      jPanel1.add(right, BorderLayout.EAST);
      JLabel left = new JLabel();
      left.setPreferredSize(new Dimension(3, 3));
      jPanel1.add(left, BorderLayout.WEST);
      JPanel center = new JPanel();
      center.setLayout(new GridLayout(3, 3, 5, 5));
      jPanel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), Localization.getString("MainControlsFrame.Volume"), TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, new Font("Dialog", Font.BOLD, 12), new Color(51, 51, 51))); //$NON-NLS-1$ //$NON-NLS-2$
      center.add(jLabel, null);
      center.add(getVolDownButton(), null);
      center.add(getVolUpButton(), null);
      center.add(jLabel1, null);
      center.add(getSoundsDownButton(), null);
      center.add(getSoundsUpButton(), null);
      center.add(jLabel2, null);
      center.add(getMusicDownButton(), null);
      center.add(getMusicUpButton(), null);
      jPanel1.add(center, BorderLayout.CENTER);
    }
    return jPanel1;
  }
  
  private final int volumeRepeat = 100;

  /**
   * This method initializes volDownButton	
   * 	
   * @return javax.swing.JButton	
   */
  private JButton getVolDownButton() {
    if (volDownButton == null) {
      volDownButton = new JButton();
      volDownButton.setModel(new RepeatingButtonModel(volumeRepeat));
      volDownButton.setAction(new KeyAction(getKeyStroke(KeyEvent.VK_DOWN)));
      volDownButton.setText(Localization.getString("MainControlsFrame.Down")); //$NON-NLS-1$
    }
    return volDownButton;
  }

  /**
   * This method initializes volUpButton	
   * 	
   * @return javax.swing.JButton	
   */
  private JButton getVolUpButton() {
    if (volUpButton == null) {
      volUpButton = new JButton();
      volUpButton.setModel(new RepeatingButtonModel(volumeRepeat));
      volUpButton.setAction(new KeyAction(getKeyStroke(KeyEvent.VK_UP)));
      volUpButton.setText(Localization.getString("MainControlsFrame.Up")); //$NON-NLS-1$
    }
    return volUpButton;
  }
  
  private final int soundsAndMusicRepeat = 50;

  /**
   * This method initializes soundsDownButton	
   * 	
   * @return javax.swing.JButton	
   */
  private JButton getSoundsDownButton() {
    if (soundsDownButton == null) {
      soundsDownButton = new JButton();
      soundsDownButton.setModel(new RepeatingButtonModel(soundsAndMusicRepeat));
      soundsDownButton.setAction(new KeyAction(getKeyStroke(KeyEvent.VK_PAGE_DOWN)));
      soundsDownButton.setText(Localization.getString("MainControlsFrame.Down")); //$NON-NLS-1$
    }
    return soundsDownButton;
  }

  /**
   * This method initializes soundsUpButton	
   * 	
   * @return javax.swing.JButton	
   */
  private JButton getSoundsUpButton() {
    if (soundsUpButton == null) {
      soundsUpButton = new JButton();
      soundsUpButton.setModel(new RepeatingButtonModel(soundsAndMusicRepeat));
      soundsUpButton.setAction(new KeyAction(getKeyStroke(KeyEvent.VK_PAGE_UP)));
      soundsUpButton.setText(Localization.getString("MainControlsFrame.Up")); //$NON-NLS-1$
    }
    return soundsUpButton;
  }

  /**
   * This method initializes musicDownButton	
   * 	
   * @return javax.swing.JButton	
   */
  private JButton getMusicDownButton() {
    if (musicDownButton == null) {
      musicDownButton = new JButton();
      musicDownButton.setModel(new RepeatingButtonModel(soundsAndMusicRepeat));
      musicDownButton.setAction(new KeyAction(getKeyStroke(KeyEvent.VK_DELETE)));
      musicDownButton.setText(Localization.getString("MainControlsFrame.Down")); //$NON-NLS-1$
    }
    return musicDownButton;
  }

  /**
   * This method initializes musicUpButton	
   * 	
   * @return javax.swing.JButton	
   */
  private JButton getMusicUpButton() {
    if (musicUpButton == null) {
      musicUpButton = new JButton();
      musicUpButton.setModel(new RepeatingButtonModel(soundsAndMusicRepeat));
      musicUpButton.setAction(new KeyAction(getKeyStroke(KeyEvent.VK_INSERT)));
      musicUpButton.setText(Localization.getString("MainControlsFrame.Up")); //$NON-NLS-1$
    }
    return musicUpButton;
  }

  public int getHierarchyPosition() {
    return 1;
  }
}  //  @jve:decl-index=0:visual-constraint="10,10"
