/*
 Copyright (c) 2009 [Joerg Ruedenauer]
 
 This file is part of RPGSoundController.

 RPGSoundController is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 RPGSoundController is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with RPGSoundController; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package rpgsoundcontroller.gui;

import java.awt.Dimension;
import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.util.List;
import java.util.prefs.Preferences;

import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import rpgsoundcontroller.messages.IMessageListener;
import rpgsoundcontroller.messages.Message;
import rpgsoundcontroller.messages.Messages;
import rpgsoundcontroller.util.Localization;

class MessageFrame extends SubFrame implements IMessageListener {

  private JPanel jContentPane = null;
  private JScrollPane jScrollPane = null;
  private JTextArea textArea = null;

  /**
   * This method initializes 
   * 
   */
  public MessageFrame() {
  	super(Localization.getString("MessageFrame.Messages")); //$NON-NLS-1$
  	initialize();
  	addExistingMessages();
  	Messages.getInstance().addObserver(this);
  }
  
  public void dispose() {
    Messages.getInstance().removeObserver(this);
    super.dispose();
  }

  private void addExistingMessages() {
    List<Message> messages = Messages.getInstance().getMessages();
    StringBuilder builder = new StringBuilder();
    int messageLevel = Preferences.userNodeForPackage(MessageFrame.class).getInt("MessageLevel", 1); //$NON-NLS-1$
    for (Message message : messages) {
      if (message.getType().ordinal() <= messageLevel)
      builder.append(messageToString(message));
    }
    getTextArea().setText(builder.toString());
  }
  

  private String messageToString(Message message) {
    StringBuilder builder = new StringBuilder();
    builder.append(message.getType());
    builder.append(": "); //$NON-NLS-1$
    builder.append(message.getMessage());
    builder.append("\n"); //$NON-NLS-1$
    return builder.toString();
  }

  /**
   * This method initializes this
   * 
   */
  private void initialize() {
        this.setSize(new Dimension(363, 246));
        this.setContentPane(getJContentPane());
        this.setTitle(Localization.getString("MessageFrame.Messages")); //$NON-NLS-1$
        this.setDefaultCloseOperation(javax.swing.JFrame.DISPOSE_ON_CLOSE);             		
  }

  /**
   * This method initializes jContentPane	
   * 	
   * @return javax.swing.JPanel	
   */
  private JPanel getJContentPane() {
    if (jContentPane == null) {
      jContentPane = new JPanel();
      jContentPane.setLayout(new BorderLayout());
      jContentPane.add(getJScrollPane(), BorderLayout.CENTER);
    }
    return jContentPane;
  }

  /**
   * This method initializes jScrollPane	
   * 	
   * @return javax.swing.JScrollPane	
   */
  private JScrollPane getJScrollPane() {
    if (jScrollPane == null) {
      jScrollPane = new JScrollPane(getTextArea());
      //jScrollPane.setViewportView(getTextArea());
    }
    return jScrollPane;
  }

  /**
   * This method initializes textArea	
   * 	
   * @return javax.swing.JTextArea	
   */
  private JTextArea getTextArea() {
    if (textArea == null) {
      textArea = new JTextArea();
    }
    return textArea;
  }

  public void messageAdded(Message message) {
    int messageLevel = Preferences.userNodeForPackage(MessageFrame.class).getInt("MessageLevel", 1); //$NON-NLS-1$
    if (message.getType().ordinal() > messageLevel) return;
    String oldText = getTextArea().getText();
    String newText = oldText + messageToString(message);
    getTextArea().setText(newText);
  }

  public int getHierarchyPosition() {
    return 1;
  }
}  //  @jve:decl-index=0:visual-constraint="10,10"
