/*
 Copyright (c) 2009 [Joerg Ruedenauer]
 
 This file is part of RPGSoundController.

 RPGSoundController is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 RPGSoundController is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with RPGSoundController; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package rpgsoundcontroller.gui;

import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.KeyStroke;

import rpgsoundcontroller.control.CommandListener;
import rpgsoundcontroller.control.Control;
import rpgsoundcontroller.control.ModeCommandAction;
import rpgsoundcontroller.data.Command;
import rpgsoundcontroller.data.Mode;

final class ModeFrame extends CommandsFrame implements CommandListener {
  
  private KeyStroke modeKey;
  
  private List<AbstractButton> pushButtons;
  
  private List<AbstractButton> getPushButtons() {
    if (pushButtons == null) {
      pushButtons = new ArrayList<AbstractButton>();
    }
    return pushButtons;
  }

  public ModeFrame(Mode mode) {
    super(mode.getTitle(), mode.getCommands());
    modeKey = mode.getKeyStroke();
    initialize();
    Control.getInstance().addObserver(this);
  }
  
  public void dispose() {
    Control.getInstance().removeObserver(this);
    super.dispose();
  }

  protected AbstractAction createAction(Command command, AbstractButton button) {
    if (command.isContinuous()) {
      getPushButtons().add(button);
    }
    return new ModeCommandAction(modeKey, command.getKeyStroke(), command.stopsAllOthers());
  }
  
  protected void deselectButton(AbstractButton button) {
    // generate an action event so that the previous command is deactivated
    if (button.isSelected()) {
      button.doClick();
    }
  }

  public int getHierarchyPosition() {
    return 2;
  }

  public void allStopped(Object source) {
    for (AbstractButton button : getPushButtons()) {
      if (button != source) {
        button.setSelected(false);
      }
    }
  }
}
