/*
 Copyright (c) 2009 [Joerg Ruedenauer]
 
 This file is part of RPGSoundController.

 RPGSoundController is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 RPGSoundController is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with RPGSoundController; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package rpgsoundcontroller.gui;

import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.AbstractButton;

import rpgsoundcontroller.data.Command;
import rpgsoundcontroller.util.Localization;

final class ModesFrame extends CommandsFrame {

  public ModesFrame(List<Command> modes) {
    super(Localization.getString("ModesFrame.Modes"), modes); //$NON-NLS-1$
    initialize();
  }
  
  private static class SelectModeAction extends AbstractAction {
    
    private String title;
    
    public SelectModeAction(Command command) {
      this.title = command.getTitle();
    }

    public void actionPerformed(ActionEvent arg0) {
      if (ModeFrames.getInstance().isFrameOpen(title)) {
        ModeFrames.getInstance().closeFrame(title);
      }
      else {
        ModeFrames.getInstance().openFrame(title);
      }
    }
  }

  public void closeFrame(SubFrame frame) {
    // nothing, handled by ModeFrames
  }
  
  public void openFrame(String name, Rectangle bounds) {
    if (name.equals(getTitle())) {
      setBounds(bounds);
      return;
    }
    // nothing, handled by ModeFrames
  }
  
  protected AbstractAction createAction(Command command, AbstractButton button) {
    addButton(command.getTitle(), button);
    button.setSelected(ModeFrames.getInstance().isFrameOpen(command.getTitle()));
    return new SelectModeAction(command);
  }
  
  protected void deselectButton(AbstractButton button) {
    // don't do anything, mode windows can be opened in parallel
  }

  public int getHierarchyPosition() {
    return 1;
  }
}
