/*
 Copyright (c) 2009 [Joerg Ruedenauer]
 
 This file is part of RPGSoundController.

 RPGSoundController is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 RPGSoundController is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with RPGSoundController; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package rpgsoundcontroller.gui.util;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultButtonModel;
import javax.swing.Timer;

public class RepeatingButtonModel extends DefaultButtonModel implements ActionListener {

  public RepeatingButtonModel() {
    super();
    repeatTime = 150;
  }
  
  public RepeatingButtonModel(int repeatTimeMillis) {
    super();
    repeatTime = repeatTimeMillis;
  }
  
  public int getRepeatTime() {
    return repeatTime;
  }
  
  public void setRepeatTime(int newTime) {
    repeatTime = newTime;
  }
  
  public void setPressed(boolean b) {
    if((isPressed() == b) || !isEnabled()) {
      return;
    }
    if (b)
    {
      timer = new Timer(repeatTime, this);
      timer.start();
    }
    else
    {
      timer.stop();
      timer = null;
    }
    super.setPressed(b);
  }
  
  public void actionPerformed(ActionEvent arg0) {
    if (isEnabled() && isArmed()) 
      fireEvent(arg0);
  }
    
  protected void fireEvent(ActionEvent timerEvent) {
    ActionEvent event = new ActionEvent(this, ActionEvent.ACTION_PERFORMED, 
        getActionCommand(), timerEvent.getWhen(), timerEvent.getModifiers());
    fireActionPerformed(event);
  }

  private Timer timer;
  
  private int repeatTime;
}
